@@ -57,6 +57,8 @@
 #define TIMESTAMP_INTERVAL 300
 
 #define MAX_LINES 800
+#define MAX_SCROLL_TIME 0.4 /* seconds */
+#define SCROLL_DELAY 33     /* milliseconds */
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CHAT_VIEW, GossipChatViewPriv))
 
@@ -77,6 +79,8 @@ struct _GossipChatViewPriv {
 	BlockType      last_block_type;
 
 	gboolean       allow_scrolling;
+	guint          scroll_timeout;
+	GTimer        *scroll_time;
 	gboolean       is_group_chat;
 
 	GtkTextMark   *find_mark_previous;
@@ -343,6 +347,12 @@ chat_view_finalize (GObject *object)
 	if (priv->last_contact) {
 		g_object_unref (priv->last_contact);
 	}
+	if (priv->scroll_time) {
+		g_timer_destroy (priv->scroll_time);
+	}
+	if (priv->scroll_timeout) {
+		g_source_remove (priv->scroll_timeout);
+	}
 
 	G_OBJECT_CLASS (gossip_chat_view_parent_class)->finalize (object);
 }
@@ -372,7 +382,10 @@ chat_view_size_allocate (GtkWidget     *widget,
 	GTK_WIDGET_CLASS (gossip_chat_view_parent_class)->size_allocate (widget, alloc);
 
 	if (down) {
-		gossip_chat_view_scroll_down (GOSSIP_CHAT_VIEW (widget));
+		GtkAdjustment *adj;
+
+		adj = GTK_TEXT_VIEW (widget)->vadjustment;
+		gtk_adjustment_set_value (adj, adj->upper - adj->page_size);
 	}
 }
 
@@ -1049,7 +1062,6 @@ chat_view_maybe_append_fancy_header (GossipChatView *view,
 {
 	GossipChatViewPriv *priv;
 	GossipContact      *sender;
-	GossipContact      *my_contact;
 	const gchar        *name;
 	gboolean            header;
 	GtkTextIter         iter;
@@ -1065,9 +1077,8 @@ chat_view_maybe_append_fancy_header (GossipChatView *view,
 	priv = GET_PRIV (view);
 
 	sender = gossip_message_get_sender (msg);
-	my_contact = gossip_contact_get_user (sender);
 	name = gossip_contact_get_name (sender);
-	from_self = gossip_contact_equal (sender, my_contact);
+	from_self = gossip_contact_is_user (sender);
 
 	gossip_debug (DEBUG_DOMAIN, "Maybe add fancy header");
 
@@ -1176,7 +1187,6 @@ chat_view_append_irc_action (GossipChatView *view,
 			     GossipMessage  *msg)
 {
 	GossipChatViewPriv *priv;
-	GossipContact      *my_contact;
 	GossipContact      *sender;
 	const gchar        *name;
 	GtkTextIter         iter;
@@ -1189,11 +1199,9 @@ chat_view_append_irc_action (GossipChatView *view,
 	gossip_debug (DEBUG_DOMAIN, "Add IRC action");
 
 	sender = gossip_message_get_sender (msg);
-	my_contact = gossip_contact_get_user (sender);
 	name = gossip_contact_get_name (sender);
 
-	/* Skip the "/me ". */
-	if (gossip_contact_equal (sender, my_contact)) {
+	if (gossip_contact_is_user (sender)) {
 		tag = "irc-action-self";
 	} else {
 		tag = "irc-action-other";
@@ -1226,7 +1234,6 @@ chat_view_append_fancy_action (GossipChatView *view,
 {
 	GossipChatViewPriv *priv;
 	GossipContact      *sender;
-	GossipContact      *my_contact;
 	const gchar        *name;
 	const gchar        *body;
 	GtkTextIter         iter;
@@ -1239,10 +1246,9 @@ chat_view_append_fancy_action (GossipChatView *view,
 	gossip_debug (DEBUG_DOMAIN, "Add fancy action");
 
 	sender = gossip_message_get_sender (msg);
-	my_contact = gossip_contact_get_user (sender);
 	name = gossip_contact_get_name (sender);
 
-	if (gossip_contact_equal (sender, my_contact)) {
+	if (gossip_contact_is_user (sender)) {
 		tag = "fancy-action-self";
 		line_tag = "fancy-line-self";
 	} else {
@@ -1270,7 +1276,6 @@ chat_view_append_irc_message (GossipChatView *view,
 {
 	GossipChatViewPriv *priv;
 	GossipContact      *sender;
-	GossipContact      *my_contact;
 	const gchar        *name;
 	const gchar        *body;
 	const gchar        *nick_tag;
@@ -1284,10 +1289,9 @@ chat_view_append_irc_message (GossipChatView *view,
 
 	body = gossip_message_get_body (msg);
 	sender = gossip_message_get_sender (msg);
-	my_contact = gossip_contact_get_user (sender);
 	name = gossip_contact_get_name (sender);
 
-	if (gossip_contact_equal (sender, my_contact)) {
+	if (gossip_contact_is_user (sender)) {
 		nick_tag = "irc-nick-self";
 		body_tag = "irc-body-self";
 	} else {
@@ -1328,16 +1332,14 @@ chat_view_append_fancy_message (GossipChatView *view,
 {
 	GossipChatViewPriv *priv;
 	GossipContact      *sender;
-	GossipContact      *my_contact;
 	const gchar        *body;
 	const gchar        *tag;
 
 	priv = GET_PRIV (view);
 
 	sender = gossip_message_get_sender (msg);
-	my_contact = gossip_contact_get_user (sender);
 
-	if (gossip_contact_equal (sender, my_contact)) {
+	if (gossip_contact_is_user (sender)) {
 		tag = "fancy-body-self";
 	} else {
 		tag = "fancy-body-other";
@@ -1431,7 +1433,6 @@ gossip_chat_view_append_message (GossipChatView *view,
 {
 	GossipChatViewPriv *priv;
 	GossipContact      *sender;
-	GossipContact      *my_contact;
 	const gchar        *body;
 	gboolean            scroll_down;
 
@@ -1470,14 +1471,12 @@ gossip_chat_view_append_message (GossipChatView *view,
 		}
 	}
 
-	my_contact = gossip_contact_get_user (sender);
-
 	/* Reset the last inserted contact. */
 	if (priv->last_contact) {
 		g_object_unref (priv->last_contact);
 	}
 
-	if (gossip_contact_equal (my_contact, sender)) {
+	if (gossip_contact_is_user (sender)) {
 		priv->last_block_type = BLOCK_TYPE_SELF;
 		priv->last_contact = NULL;
 	} else {
@@ -1629,13 +1628,38 @@ gossip_chat_view_scroll (GossipChatView *view,
 		      allow_scrolling ? "enabled" : "disabled");
 }
 
+/* Code stolen from pidgin/gtkimhtml.c */
+static gboolean
+chat_view_scroll_cb (GossipChatView *view)
+{
+	GossipChatViewPriv *priv;
+	GtkAdjustment      *adj;
+	gdouble             max_val;
+
+	priv = GET_PRIV (view);
+	adj = GTK_TEXT_VIEW (view)->vadjustment;
+	max_val = adj->upper - adj->page_size;
+
+	g_return_val_if_fail (priv->scroll_time != NULL, FALSE);
+
+	if (g_timer_elapsed (priv->scroll_time, NULL) > MAX_SCROLL_TIME) {
+		/* time's up. jump to the end and kill the timer */
+		gtk_adjustment_set_value (adj, max_val);
+		g_timer_destroy (priv->scroll_time);
+		priv->scroll_time = NULL;
+		priv->scroll_timeout = 0;
+		return FALSE;
+	}
+
+	/* scroll by 1/3rd the remaining distance */
+	gtk_adjustment_set_value (adj, gtk_adjustment_get_value (adj) + ((max_val - gtk_adjustment_get_value (adj)) / 3));
+	return TRUE;
+}
+
 void
 gossip_chat_view_scroll_down (GossipChatView *view)
 {
 	GossipChatViewPriv *priv;
-	GtkTextBuffer      *buffer;
-	GtkTextIter         iter;
-	GtkTextMark        *mark;
 
 	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
 
@@ -1647,22 +1671,16 @@ gossip_chat_view_scroll_down (GossipChatView *view)
 
 	gossip_debug (DEBUG_DOMAIN, "Scrolling down");
 
-	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
-
-	gtk_text_buffer_get_end_iter (buffer, &iter);
-	mark = gtk_text_buffer_create_mark (buffer,
-					    NULL,
-					    &iter,
-					    FALSE);
-
-	gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (view),
-				      mark,
-				      0.0,
-				      FALSE,
-				      0,
-				      0);
-
-	gtk_text_buffer_delete_mark (buffer, mark);
+	if (priv->scroll_time) {
+		g_timer_reset (priv->scroll_time);
+	} else {
+		priv->scroll_time = g_timer_new();
+	}
+	if (!priv->scroll_timeout) {
+		priv->scroll_timeout = g_timeout_add (SCROLL_DELAY,
+						      (GSourceFunc) chat_view_scroll_cb,
+						      view);
+	}
 }
 
 gboolean
