@@ -33,7 +33,6 @@
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
 #include <libempathy/empathy-contact-manager.h>
 #include <libempathy/empathy-log-manager.h>
@@ -47,7 +46,7 @@
 #include "gossip-geometry.h"
 #include "gossip-preferences.h"
 #include "gossip-spell.h"
-//#include "gossip-spell-dialog.h"
+#include "gossip-spell-dialog.h"
 #include "gossip-ui-utils.h"
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CHAT, GossipChatPriv))
@@ -341,9 +340,8 @@ static void
 chat_send (GossipChat  *chat,
 	   const gchar *msg)
 {
-	GossipChatPriv    *priv;
-	GossipMessage     *message;
-	GossipContact     *own_contact;
+	GossipChatPriv *priv;
+	GossipMessage  *message;
 
 	priv = GET_PRIV (chat);
 
@@ -361,9 +359,7 @@ chat_send (GossipChat  *chat,
 	/* FIXME: add here something to let group/privrate chat handle
 	 *        some special messages */
 
-	own_contact = empathy_contact_manager_get_user (priv->manager, chat->account);
 	message = gossip_message_new (msg);
-	gossip_message_set_sender (message, own_contact);
 
 	empathy_tp_chat_send (priv->tp_chat, message);
 
@@ -415,6 +411,7 @@ chat_message_received_cb (EmpathyTpChat *tp_chat,
 	if (timestamp >= priv->time_joined) {
 		empathy_log_manager_add_message (priv->log_manager,
 						 gossip_chat_get_id (chat),
+						 gossip_chat_is_group_chat (chat),
 						 message);
 	}
 
@@ -936,10 +933,10 @@ static void
 chat_text_check_word_spelling_cb (GtkMenuItem     *menuitem,
 				  GossipChatSpell *chat_spell)
 {
-/*FIXME:	gossip_spell_dialog_show (chat_spell->chat,
+	gossip_spell_dialog_show (chat_spell->chat,
 				  chat_spell->start,
 				  chat_spell->end,
-				  chat_spell->word);*/
+				  chat_spell->word);
 }
 
 static GossipChatSpell *
@@ -1035,14 +1032,12 @@ chat_state_changed_cb (EmpathyTpChat             *tp_chat,
 		       GossipChat                *chat)
 {
 	GossipChatPriv *priv;
-	GossipContact  *own_contact;
 	GList          *l;
 	gboolean        was_composing;
 
 	priv = GET_PRIV (chat);
 
-	own_contact = gossip_contact_get_user (contact);
-	if (gossip_contact_equal (own_contact, contact)) {
+	if (gossip_contact_is_user (contact)) {
 		/* We don't care about our own chat state */
 		return;
 	}
@@ -1112,7 +1107,8 @@ chat_add_logs (GossipChat *chat)
 	/* Add messages from last conversation */
 	messages = empathy_log_manager_get_last_messages (priv->log_manager,
 							  chat->account,
-							  gossip_chat_get_id (chat));
+							  gossip_chat_get_id (chat),
+							  gossip_chat_is_group_chat (chat));
 	num_messages  = g_list_length (messages);
 
 	for (l = messages, i = 0; l; l = l->next, i++) {
@@ -1498,7 +1494,7 @@ gossip_chat_should_play_sound (GossipChat *chat)
 gboolean
 gossip_chat_should_highlight_nick (GossipMessage *message)
 {
-	GossipContact *my_contact;
+	GossipContact *contact;
 	const gchar   *msg, *to;
 	gchar         *cf_msg, *cf_to;
 	gchar         *ch;
@@ -1515,8 +1511,12 @@ gossip_chat_should_highlight_nick (GossipMessage *message)
 		return FALSE;
 	}
 
-	my_contact = gossip_contact_get_user (gossip_message_get_sender (message));
-	to = gossip_contact_get_name (my_contact);
+	contact = gossip_message_get_receiver (message);
+	if (!contact || !gossip_contact_is_user (contact)) {
+		return FALSE;
+	}
+
+	to = gossip_contact_get_name (contact);
 	if (!to) {
 		return FALSE;
 	}
