@@ -914,7 +914,7 @@ preferences_destroy_cb (GtkWidget         *widget,
 }
 
 GtkWidget *
-gossip_preferences_show (void)
+gossip_preferences_show (GtkWindow *parent)
 {
 	static GossipPreferences *preferences;
 	GladeXML                 *glade;
@@ -973,6 +973,11 @@ gossip_preferences_show (void)
 		gtk_widget_show (page);
 	}
 
+	if (parent) {
+		gtk_window_set_transient_for (GTK_WINDOW (preferences->dialog),
+					      GTK_WINDOW (parent));
+	}
+
 	gtk_widget_show (preferences->dialog);
 
 	return preferences->dialog;
