@@ -33,11 +33,10 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
-#include <libgnome/libgnome.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 
 #include <libmissioncontrol/mc-profile.h>
 
-#include <libempathy/gossip-paths.h>
 #include <libempathy/gossip-debug.h>
 
 #include "gossip-ui-utils.h"
@@ -63,7 +62,7 @@ get_glade_file (const gchar *filename,
 	const char *name;
 	GtkWidget **widget_ptr;
 
-	path = gossip_paths_get_glade_path (filename);
+	path = g_build_filename (DATADIR, "empathy", filename, NULL);
 	gui = glade_xml_new (path, root, domain);
 	g_free (path);
 
@@ -384,15 +383,12 @@ gossip_icon_name_for_contact (GossipContact *contact)
 			      EMPATHY_IMAGE_OFFLINE);
 
 	presence = gossip_contact_get_presence (contact);
-
 	if (presence) {
 		return gossip_icon_name_for_presence (presence);
 	}
 
 	subscription = gossip_contact_get_subscription (contact);
-
-	if (subscription != GOSSIP_SUBSCRIPTION_BOTH &&
-	    subscription != GOSSIP_SUBSCRIPTION_TO) {
+	if (!(subscription & GOSSIP_SUBSCRIPTION_FROM)) {
 		return EMPATHY_IMAGE_PENDING;
 	}
 
@@ -1255,6 +1251,21 @@ gossip_window_present (GtkWindow *window,
 	}
 }
 
+GtkWindow *
+gossip_get_toplevel_window (GtkWidget *widget)
+{
+	GtkWidget *toplevel;
+
+	g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
+
+	toplevel = gtk_widget_get_toplevel (widget);
+	if (GTK_IS_WINDOW (toplevel)) {
+		return GTK_WINDOW (toplevel);
+	}
+
+	return NULL;
+}
+
 /* The URL opening code can't handle schemeless strings, so we try to be
  * smart and add http if there is no scheme or doesn't look like a mail
  * address. This should work in most cases, and let us click on strings
@@ -1279,14 +1290,15 @@ fixup_url (const gchar *url)
 void
 gossip_url_show (const char *url)
 {
-	gchar  *real_url;
-	GError *error = NULL;
+	gchar          *real_url;
+	GnomeVFSResult  res;
 
 	real_url = fixup_url (url);
-	gnome_url_show (real_url, &error);
-	if (error) {
-		g_warning ("Couldn't show URL:'%s'", real_url);
-		g_error_free (error);
+	res = gnome_vfs_url_show (real_url);
+	if (res != GNOME_VFS_OK) {
+		gossip_debug (DEBUG_DOMAIN, "Couldn't show URL %s: %s",
+			      real_url,
+			      gnome_vfs_result_to_string (res));
 	}
 
 	g_free (real_url);
@@ -1314,13 +1326,6 @@ gossip_link_button_new (const gchar *url,
 	return gtk_link_button_new_with_label (url, title);
 }
 
-/* FIXME: Do this in a proper way at some point, probably in GTK+? */
-void
-gossip_window_set_default_icon_name (const gchar *name)
-{
-	gtk_window_set_default_icon_name (name);
-}
-
 void
 gossip_toggle_button_set_state_quietly (GtkWidget *widget,
 					GCallback  callback,
