@@ -57,6 +57,9 @@ struct _EmpathyContactListIface {
 					      const gchar        *message);
 	GList *         (*get_members)       (EmpathyContactList *list);
 	GList *         (*get_local_pending) (EmpathyContactList *list);
+	void            (*process_pending)   (EmpathyContactList *list,
+					      GossipContact      *contact,
+					      gboolean            accept);
 };
 
 GType                   empathy_contact_list_get_type          (void) G_GNUC_CONST;
@@ -74,6 +77,9 @@ void                    empathy_contact_list_remove            (EmpathyContactLi
 								const gchar            *message);
 GList *                 empathy_contact_list_get_members       (EmpathyContactList     *list);
 GList *                 empathy_contact_list_get_local_pending (EmpathyContactList     *list);
+void                    empathy_contact_list_process_pending   (EmpathyContactList     *list,
+								GossipContact          *contact,
+								gboolean                accept);
 
 G_END_DECLS
 
