@@ -22,6 +22,8 @@
 
 #include <config.h>
 
+#include <string.h>
+
 #include <glib/gi18n.h>
 #include <dbus/dbus-glib.h>
 
@@ -39,19 +41,25 @@
 /* Number of seconds before entering extended autoaway. */
 #define EXT_AWAY_TIME (30*60)
 
-enum {
-	LAST_SIGNAL
-};
+typedef enum {
+	NM_STATE_UNKNOWN = 0,
+	NM_STATE_ASLEEP,
+	NM_STATE_CONNECTING,
+	NM_STATE_CONNECTED,
+	NM_STATE_DISCONNECTED
+} NMState;
 
 struct _EmpathyIdlePriv {
 	MissionControl *mc;
 	DBusGProxy     *gs_proxy;
+	DBusGProxy     *nm_proxy;
 	gboolean        is_idle;
 	McPresence      state;
-	McPresence      slack_state;
+	McPresence      flash_state;
 	gchar          *status;
 	McPresence      saved_state;
 	gchar          *saved_status;
+	gboolean        nm_connected;
 	guint           ext_away_timeout;
 };
 
@@ -72,6 +80,9 @@ static void     idle_presence_changed_cb     (MissionControl   *mc,
 static void     idle_session_idle_changed_cb (DBusGProxy       *gs_proxy,
 					      gboolean          is_idle,
 					      EmpathyIdle      *idle);
+static void     idle_nm_state_change_cb      (DBusGProxy       *proxy,
+					      guint             state,
+					      EmpathyIdle      *idle);
 static void     idle_ext_away_start          (EmpathyIdle      *idle);
 static void     idle_ext_away_stop           (EmpathyIdle      *idle);
 static gboolean idle_ext_away_cb             (EmpathyIdle      *idle);
@@ -80,7 +91,7 @@ enum {
 	PROP_0,
 	PROP_STATE,
 	PROP_STATUS,
-	PROP_SLACK_STATE
+	PROP_FLASH_STATE
 };
 
 G_DEFINE_TYPE (EmpathyIdle, empathy_idle, G_TYPE_OBJECT)
@@ -111,10 +122,10 @@ empathy_idle_class_init (EmpathyIdleClass *klass)
 							      NULL,
 							      G_PARAM_READWRITE));
 	g_object_class_install_property (object_class,
-					 PROP_SLACK_STATE,
-					 g_param_spec_uint ("slack-state",
-							    "slack-state",
-							    "slack-state",
+					 PROP_FLASH_STATE,
+					 g_param_spec_uint ("flash-state",
+							    "flash-state",
+							    "flash-state",
 							    MC_PRESENCE_UNSET,
 							    LAST_MC_PRESENCE,
 							    MC_PRESENCE_UNSET,
@@ -127,32 +138,58 @@ static void
 empathy_idle_init (EmpathyIdle *idle)
 {
 	EmpathyIdlePriv *priv;
+	DBusGConnection *system_bus;
+	GError          *error = NULL;
 
 	priv = GET_PRIV (idle);
 
 	priv->is_idle = FALSE;
 	priv->mc = gossip_mission_control_new ();
 	priv->state = mission_control_get_presence_actual (priv->mc, NULL);
-	priv->status = mission_control_get_presence_message_actual (priv->mc, NULL);
-	priv->gs_proxy = dbus_g_proxy_new_for_name (tp_get_bus (),
-						    "org.gnome.ScreenSaver",
-						    "/org/gnome/ScreenSaver",
-						    "org.gnome.ScreenSaver");
-	if (!priv->gs_proxy) {
-		gossip_debug (DEBUG_DOMAIN, "Failed to get gs proxy");
-		return;
-	}
+	idle_presence_changed_cb (priv->mc, priv->state, idle);
 
-	dbus_g_proxy_add_signal (priv->gs_proxy, "SessionIdleChanged",
-				 G_TYPE_BOOLEAN,
-				 G_TYPE_INVALID);
-	dbus_g_proxy_connect_signal (priv->gs_proxy, "SessionIdleChanged",
-				     G_CALLBACK (idle_session_idle_changed_cb),
-				     idle, NULL);
 	dbus_g_proxy_connect_signal (DBUS_G_PROXY (priv->mc),
 				     "PresenceStatusActual",
 				     G_CALLBACK (idle_presence_changed_cb),
 				     idle, NULL);
+
+	priv->gs_proxy = dbus_g_proxy_new_for_name (tp_get_bus (),
+						    "org.gnome.ScreenSaver",
+						    "/org/gnome/ScreenSaver",
+						    "org.gnome.ScreenSaver");
+	if (priv->gs_proxy) {
+		dbus_g_proxy_add_signal (priv->gs_proxy, "SessionIdleChanged",
+					 G_TYPE_BOOLEAN,
+					 G_TYPE_INVALID);
+		dbus_g_proxy_connect_signal (priv->gs_proxy, "SessionIdleChanged",
+					     G_CALLBACK (idle_session_idle_changed_cb),
+					     idle, NULL);
+	} else {
+		gossip_debug (DEBUG_DOMAIN, "Failed to get gs proxy");
+	}
+
+
+	system_bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
+	if (!system_bus) {
+		gossip_debug (DEBUG_DOMAIN, "Failed to get system bus: %s",
+			      error ? error->message : "No error given");
+	} else {
+		priv->nm_proxy = dbus_g_proxy_new_for_name (system_bus,
+							    "org.freedesktop.NetworkManager",
+							    "/org/freedesktop/NetworkManager",
+							    "org.freedesktop.NetworkManager");
+	}
+	if (priv->nm_proxy) {
+		dbus_g_proxy_add_signal (priv->nm_proxy, "StateChange",
+					 G_TYPE_UINT, G_TYPE_INVALID);
+		dbus_g_proxy_connect_signal (priv->nm_proxy, "StateChange",
+					     G_CALLBACK (idle_nm_state_change_cb),
+					     idle, NULL);
+	} else {
+		gossip_debug (DEBUG_DOMAIN, "Failed to get nm proxy");
+	}
+	/* FIXME: get value */
+	priv->nm_connected = TRUE;
 }
 
 static void
@@ -192,8 +229,8 @@ idle_get_property (GObject    *object,
 	case PROP_STATUS:
 		g_value_set_string (value, empathy_idle_get_status (idle));
 		break;
-	case PROP_SLACK_STATE:
-		g_value_set_uint (value, empathy_idle_get_slack_state (idle));
+	case PROP_FLASH_STATE:
+		g_value_set_uint (value, empathy_idle_get_flash_state (idle));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
@@ -220,8 +257,8 @@ idle_set_property (GObject      *object,
 	case PROP_STATUS:
 		empathy_idle_set_status (idle, g_value_get_string (value));
 		break;
-	case PROP_SLACK_STATE:
-		empathy_idle_set_slack_state (idle, g_value_get_uint (value));
+	case PROP_FLASH_STATE:
+		empathy_idle_set_flash_state (idle, g_value_get_uint (value));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
@@ -262,10 +299,7 @@ empathy_idle_set_state (EmpathyIdle *idle,
 
 	priv = GET_PRIV (idle);
 
-	mission_control_set_presence (priv->mc,
-				      state,
-				      priv->status,
-				      NULL, NULL);
+	empathy_idle_set_presence (idle, state, priv->status);
 }
 
 const gchar *
@@ -275,6 +309,10 @@ empathy_idle_get_status (EmpathyIdle *idle)
 
 	priv = GET_PRIV (idle);
 
+	if (!priv->status) {
+		return gossip_presence_state_get_default_status (priv->state);
+	}
+
 	return priv->status;
 }
 
@@ -286,33 +324,62 @@ empathy_idle_set_status (EmpathyIdle *idle,
 
 	priv = GET_PRIV (idle);
 
-	mission_control_set_presence (priv->mc,
-				      priv->state,
-				      status,
-				      NULL, NULL);
+	empathy_idle_set_presence (idle, priv->state, status);
 }
 
 McPresence
-empathy_idle_get_slack_state (EmpathyIdle *idle)
+empathy_idle_get_flash_state (EmpathyIdle *idle)
 {
 	EmpathyIdlePriv *priv;
 
 	priv = GET_PRIV (idle);
 
-	return priv->slack_state;
+	return priv->flash_state;
 }
 
 void
-empathy_idle_set_slack_state (EmpathyIdle *idle,
+empathy_idle_set_flash_state (EmpathyIdle *idle,
 			      McPresence   state)
 {
 	EmpathyIdlePriv *priv;
 
 	priv = GET_PRIV (idle);
 
-	priv->slack_state = state;
+	priv->flash_state = state;
 
-	g_object_notify (G_OBJECT (idle), "slack-state");
+	if (state == MC_PRESENCE_UNSET) {
+	}
+
+	g_object_notify (G_OBJECT (idle), "flash-state");
+}
+
+void
+empathy_idle_set_presence (EmpathyIdle *idle,
+			   McPresence   state,
+			   const gchar *status)
+{
+	EmpathyIdlePriv *priv;
+	const gchar     *default_status;
+
+	priv = GET_PRIV (idle);
+
+	if (!priv->nm_connected) {
+		g_free (priv->saved_status);
+		priv->saved_state = state;
+		priv->saved_status = g_strdup (status);
+		return;
+	}
+
+	/* Do not set translated default messages */
+	default_status = gossip_presence_state_get_default_status (state);
+	if (status && strcmp (status, default_status) == 0) {
+		status = NULL;
+	}
+
+	mission_control_set_presence (priv->mc,
+				      state,
+				      status,
+				      NULL, NULL);
 }
 
 static void
@@ -330,7 +397,7 @@ idle_presence_changed_cb (MissionControl *mc,
 
 	if (G_STR_EMPTY (priv->status)) {
 		g_free (priv->status);
-		priv->status = g_strdup (gossip_presence_state_get_default_status (state));
+		priv->status = NULL;
 	}
 
 	g_object_notify (G_OBJECT (idle), "state");
@@ -350,15 +417,18 @@ idle_session_idle_changed_cb (DBusGProxy  *gs_proxy,
 		      priv->is_idle ? "yes" : "no",
 		      is_idle ? "yes" : "no");
 
+	if (priv->state <= MC_PRESENCE_OFFLINE ||
+	    priv->state == MC_PRESENCE_HIDDEN) {
+		/* We are not online so nothing to do here */
+		priv->is_idle = is_idle;
+		return;
+	}
+
 	if (is_idle && !priv->is_idle) {
 		McPresence new_state;
-		/* We are now idle, set state to away */
+		/* We are now idle */
 
-		if (priv->state <= MC_PRESENCE_OFFLINE ||
-		    priv->state == MC_PRESENCE_HIDDEN) {
-			/* We are not online so nothing to do here */
-			return;
-		} else if (priv->state == MC_PRESENCE_AWAY ||
+		if (priv->state == MC_PRESENCE_AWAY ||
 			   priv->state == MC_PRESENCE_EXTENDED_AWAY) {
 			/* User set away manually, when coming back we restore
 			 * default presence. */
@@ -383,10 +453,9 @@ idle_session_idle_changed_cb (DBusGProxy  *gs_proxy,
 			      priv->saved_state,
 			      priv->saved_status);
 
-		mission_control_set_presence (priv->mc,
-					      priv->saved_state,
-					      priv->saved_status,
-					      NULL, NULL);
+		empathy_idle_set_presence (idle,
+					   priv->saved_state,
+					   priv->saved_status);
 
 		g_free (priv->saved_status);
 		priv->saved_status = NULL;
@@ -395,6 +464,38 @@ idle_session_idle_changed_cb (DBusGProxy  *gs_proxy,
 	priv->is_idle = is_idle;
 }
 
+static void
+idle_nm_state_change_cb (DBusGProxy  *proxy,
+			 guint        state,
+			 EmpathyIdle *idle)
+{
+	EmpathyIdlePriv *priv;
+
+	priv = GET_PRIV (idle);
+
+	gossip_debug (DEBUG_DOMAIN, "New network state (%d)", state);
+
+	if (state != NM_STATE_CONNECTED &&
+	    priv->state > MC_PRESENCE_OFFLINE) {
+		/* We are no more connected */
+		idle_ext_away_stop (idle);
+		g_free (priv->saved_status);
+		priv->saved_state = priv->state;
+		priv->saved_status = g_strdup (priv->status);
+
+		empathy_idle_set_state (idle, MC_PRESENCE_OFFLINE);
+		priv->nm_connected = FALSE;
+	}
+	else if (priv->state <= MC_PRESENCE_OFFLINE &&
+		 state == NM_STATE_CONNECTED) {
+		/* We are now connected */
+		priv->nm_connected = TRUE;
+		empathy_idle_set_presence (idle,
+					   priv->saved_state,
+					   priv->saved_status);
+	}
+}
+
 static void
 idle_ext_away_start (EmpathyIdle *idle)
 {
@@ -429,11 +530,7 @@ idle_ext_away_cb (EmpathyIdle *idle)
 	priv = GET_PRIV (idle);
 
 	gossip_debug (DEBUG_DOMAIN, "Going to extended autoaway");
-	mission_control_set_presence (priv->mc,
-				      MC_PRESENCE_EXTENDED_AWAY,
-				      priv->saved_status,
-				      NULL, NULL);
-
+	empathy_idle_set_state (idle, MC_PRESENCE_EXTENDED_AWAY);
 	priv->ext_away_timeout = 0;
 
 	return FALSE;
